#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2006
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# NAME: db2_start.ksh db2instance [NN] [verbose]
#
# (%W%) %E%  %U%
#
# FUNCTION: Start method for DB2 in TSA HA cluster
#
# Pre-reqs: db2_monitor.ksh, db2_stop.ksh
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

export CT_MANAGEMENT_SCOPE=2

DB2INSTANCE=${1}
NN=${2:-0}
VERBOSE=${3:-noverbose}

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi

# Functions ...
function startPartitionOnThisNode
{
   if [[ -x $INSTHOME/sqllib/bin/db2gcf ]]; then
      logger -i -p notice -t $PROGNAME "About to call db2gcf -u -p $NN, $DB2INSTANCE"
      $INSTHOME/sqllib/bin/db2gcf -t 60 -u -p ${NN?} -i ${DB2INSTANCE?}
      rcg=$?
      logger -i -p notice -t $PROGNAME "Done call to db2gcf -u -p $NN, $DB2INSTANCE: $rcg"
   fi
}

function activateDatabase
{
   Resource=db2hadr_${db?}-rs
   hn=$(hostname)
   NodeRG1=$(lsrsrc-api -s IBM.Application::'Name="'${Resource?}'" '::NodeNameList | grep -v $hn | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | tail -1)
   if [[ ! -z "$NodeRG1" ]]; then
      # HADR database ...
      OpState=$(lsrsrc-api -s IBM.Application::'Name="'${Resource?}'"&& NodeNameList={"'${NodeRG1}'"} '::OpState 2> /dev/null)
      if [[ $OpState == 1 ]]; then
         # HADR is Primary on the other side, start as standby here
         logger -i -p notice -t $PROGNAME "About to START AS STANDBY $db"
         su - ${DB2INSTANCE?} -c "db2 start hadr on db ${db?} as standby"
      else
         logger -i -p notice -t $PROGNAME "About to ACTIVATE $db"
         su - ${DB2INSTANCE?} -c "db2 activate database ${db?}"
      fi
   else
      # Not HADR database
      logger -i -p notice -t $PROGNAME "About to restart $db"
      su - ${DB2INSTANCE?} -c "db2 restart database ${db?}" &
      sleep 2
      logger -i -p notice -t $PROGNAME "Completed starting the restart of $db"
   fi
}

###################################################################
# Start main error checking...

# Do not start up instance if tmp file exists
if [[ -r /tmp/do_not_start.${DB2INSTANCE?}.${NN?} ]]; then
   logger -i -p notice -t $PROGNAME "Bypassing startup for $DB2INSTANCE,$NN..."
   return 2
fi

# Verify inputs ...
if [[ -z $DB2INSTANCE ]]; then
   logger -i -p err -t $PROGNAME "Error: Must specify instance name"
   return 2
fi

if [[ -z ${INSTHOME} ]] ; then
   userhome=~${DB2INSTANCE?}
   eval userhome=$userhome
   INSTHOME=${userhome}
fi
if [[ -z ${INSTHOME} ]] ; then
   INSTHOME=$(cat /etc/passwd | grep "^${DB2INSTANCE?}:" | cut -f 6 -d \:)
fi
if [[ -z ${INSTHOME} ]] ; then
   logger -i -p err -t $PROGNAME "There is no home directory defined for ${DB2INSTANCE?}"
   return 2
fi
INSTHOME=${INSTHOME#*=}
INSTHOME=${INSTHOME%%*( )}

###################################################################
# Done error checking, 
# Start main...
logger -i -p notice -t $PROGNAME "Entered $0, $DB2INSTANCE, $NN"


# Ensure sqllib is accessible
while :
do
   if [[ ! -d ${INSTHOME?}/sqllib ]]; then
      logger -i -p err -t $PROGNAME "$0, $DB2INSTANCE, $NN: Cannot access $INSTHOME/sqllib"
      cd ${INSTHOME?}/sqllib
      cd /
      sleep 5
   else
      break
   fi
done

# If single partition, clean IPCs first
DB2NODES_CFG=${INSTHOME?}/sqllib/db2nodes.cfg
if test -r $DB2NODES_CFG ; then
   nln=$(wc -l $DB2NODES_CFG | awk '{print $1}')
else
   nln=1
fi

logger -i -p debug -t $PROGNAME "$0, $DB2INSTANCE, $NN: Instance consists of $nln partitions"


# Check whether instance is healthy first
$PROGPATH/db2_monitor.ksh $DB2INSTANCE $NN
if [ $? -eq 1 ]; then
   logger -i -p notice -t $PROGNAME "$0, $DB2INSTANCE, $NN: Is already up"
else

   if [ $nln -eq 1 ]; then
      su - ${DB2INSTANCE} -c "$INSTHOME/sqllib/bin/db2gcf -t 60 -k -p ${NN?} -i ${DB2INSTANCE?};ipclean -a"
      kill -9 $(ps -o pid,comm -u $DB2INSTANCE | grep "db2[a-z]" | awk '{print $1}') 2> /dev/null
      kill -9 $(ps -ef | grep -v grep | grep db2chkau | awk '{print $2}') 2> /dev/null
   fi

   # Need to start this partition...
   rc=2
   rcg=2
   while [ $rc -ne 1 -o $rcg -ne 0 ]; do
      startPartitionOnThisNode
      $PROGPATH/db2_monitor.ksh $DB2INSTANCE $NN
      rc=$? 
      if [ $rc -ne 1 -o $rcg -ne 0 ]; then
         logger -i -p notice -t $PROGNAME "*************** Attempted to start partition $DB2INSTANCE,$NN and failed"

         logger -i -p notice -t $PROGNAME "Killing db2 processes : $DB2INSTANCE, $NN"
         kill -9 $(ps -o pid,comm -u $DB2INSTANCE | grep "db2[a-z]" | grep -v "db2haicu" | awk '{print $1}') 2> /dev/null

         nip=$(ps -fu $DB2INSTANCE | grep -c "ipclean" )
         if [[ $nip -eq 0 ]] ; then
            logger -i -p notice -t $PROGNAME "Removing IPCs"
            su - ${DB2INSTANCE} -c "ipclean -a"
         fi

         typeset -Z4 nnnn
         nnnn=$NN
         rm -f $INSTHOME/sqllib/ctrl/db2stst.${nnnn?}
         # Will retry start ... sleep for a bit to prevent pegging CPU
         sleep 30
   fi
done
fi

# Activate databases here
# Should only really activate for Single Partition / HADR databases
if [ $nln -eq 1 ]; then
   dbFN=/tmp/dblist.${DB2INSTANCE?}.${NN?}.out
   touch ${dbFN?}
   chmod 777 ${dbFN?}
   logger -i -p debug -t $PROGNAME "Touched ${dbFN?} ..."
   su - ${DB2INSTANCE?} -c "db2 list database directory" | grep "Database name" \
   | tr "[A-Z]" "[a-z]" | awk '{print $4}' > ${dbFN?}
   logger -i -p debug -t $PROGNAME "Created ${dbFN?} file..."
   while read db; do
     :
      # comment following line to not restart or activate database at instance start time
      activateDatabase
   done < ${dbFN?}
   logger -i -p debug -t $PROGNAME "Removing ${dbFN?} ... "
   rm -f ${dbFN?}
   logger -i -p debug -t $PROGNAME "Removed ${dbFN?}"
fi

rc=0
logger -i -p notice -t $PROGNAME "Returning $rc from $0 ( $DB2INSTANCE, $NN)"
return $rc
