#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2006
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# NAME: hadr_monitor.ksh
#
# (%W%) %E%  %U%
#
# FUNCTION: Probe for specified HADR pair alive
#
# INPUT:  hadr_monitor.ksh db2instp db2insts db2hadrdb verbose [S]
#
# OUTPUT: see probehadr() function for description of return codes
#
#-----------------------------------------------------------------------
RESOURCE_METHOD=probe

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
RT_BASEDIR=$PROGPATH
DB2HADRINSTANCE1=${1?}
DB2HADRINSTANCE2=${2?}
DB2HADRDBNAME=${3?}
VERBOSE=${4:-noverbose}
monitor_as_standby=${5:-N}
PROBE=$PROGNAME
START=
STOP=
SVC_PROBE=${RT_BASEDIR}/${PROBE:-hadr_monitor.ksh}
START_CMD=$RT_BASEDIR/${START:-hadr_start.ksh}
STOP_CMD=$RT_BASEDIR/${STOP:-hadr_stop.ksh}

export CT_MANAGEMENT_SCOPE=2

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   #  Close stdout   and   stderr
   exec  2> /dev/null
   exec  1> /dev/null
   set   +x
fi

UNAME=$(uname)
if [[ "${UNAME}" == "AIX" ]]; then
   grepFlags='-p'
else
   grepFlags='-A1'
fi


CLUSTER_APP_LABEL=db2hadr_${DB2HADRDBNAME?}


# Name the resource group and the resource
DATA_SVC_RG_NAME=${CLUSTER_APP_LABEL}-rg
DATA_SVC_R_NAME=${CLUSTER_APP_LABEL}-rs

PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

temp_snap=/tmp/.${PROGNAME?}_${DB2HADRDBNAME?}_$$
temp_snap2=/.${PROGNAME?}_${DB2HADRDBNAME?}_$$

TRAPSIGNALS="TERM KILL"
# trap 'rm -f $temp_snap ; exit $rc' 0 1 2 3 6 9 15


###########################################################
# resource_group_state_on_this_node()
#
# INPUT: ResourceClass, Resource, ResourceType, Node
# OUTPUT:rg_state [Online=1 || Offline]
#
###########################################################
resource_group_state_on_this_node()
{
   OpState=$( lsrsrc-api -s ${ResourceClass}::'Name="'${Resource}'" '"${ResourceType}"' && NodeNameList={"'${Node}'"} '::OpState 2> /dev/null)

   rg_state=${OpState?}
}

###########################################################
# set_candidate_P_instance()
#
# INPUT: DB2HADRINSTANCE1, DB2HADRINTANCE2
# OUTPUT:candidate_P_instance
#
###########################################################
set_candidate_P_instance()
{
   Node=$(hostname | tr "." " " | awk '{print $1}')

   if [[ $DB2HADRINSTANCE1 == $DB2HADRINSTANCE2 ]]; then
      candidate_P_instance=$DB2HADRINSTANCE1
   else

      RSNAME1=db2_${DB2HADRINSTANCE1?}_
      RSNAME2=db2_${DB2HADRINSTANCE2?}_

      Resource=${RSNAME1?}
      ResourceClass=IBM.Application
      NodeRG1long=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | tail -1)
      NodeRG1=$(echo $NodeRG1long | tr "." " " | awk '{print $1}')

      Resource=${RSNAME2?}
      ResourceClass=IBM.Application
      NodeRG2long=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | tail -1)
      NodeRG2=$(echo $NodeRG2long | tr "." " " | awk '{print $1}')

      if   [[ $NodeRG1 == $Node && $NodeRG2 != $Node ]]; then
         candidate_P_instance=$DB2HADRINSTANCE1
      elif [[ $NodeRG1 != $Node && $NodeRG2 == $Node ]]; then
         candidate_P_instance=$DB2HADRINSTANCE2
      else
         # either both instances are up in resource groups
         # on this node, or neither is
         # figure out the one that wants to be primary
         HADR_LOCAL_HOST1=$(su - ${DB2HADRINSTANCE1?} -c "db2 get db cfg for ${DB2HADRDBNAME?}" | grep HADR_LOCAL_HOST | awk '{print $7}' | tr "." " " | awk '{print $1}' | tail -1)
         HADR_LOCAL_HOST2=$(su - ${DB2HADRINSTANCE2?} -c "db2 get db cfg for ${DB2HADRDBNAME?}" | grep HADR_LOCAL_HOST | awk '{print $7}' | tr "." " " | awk '{print $1}' | tail -1)
         if [[ $HADR_LOCAL_HOST1 == $Node ]]; then
            candidate_P_instance=$DB2HADRINSTANCE1
         elif [[ $HADR_LOCAL_HOST2 == $Node ]]; then
            candidate_P_instance=$DB2HADRINSTANCE2
         else
            candidate_P_instance=
         fi
      fi
   fi
}

###########################################################
# probehadr_on_this_node()
#
# Probe status of HADR on the active resource on this node
#
# Return 0 if specified DB2 HADR is unknown
# Return 1 if specified DB2 HADR is online as Primary,Peer on this node
# Return 3 if specified DB2 HADR is online as Primary,non-Peer on this node
# Return 2 if specified DB2 HADR is online as Standby,Peer on this node
# Return 40 if specified DB2 HADR is online as Standby,Not Peer on this node
#
###########################################################
probehadr_on_this_node()
{
   # Is HADR process model up
   hadr_process=$(ps -u ${instance_to_monitor} -o args | egrep -i "db2hadr[ps] \(${DB2HADRDBNAME}\)")
   logger -i -p debug -t $0 "$DB2HADRDBNAME : HADR EDU : $hadr_process"

   # Use db2pd instead of SNAPSHOT if possible ...
   su - ${instance_to_monitor} -c "db2pd -hadr -db ${DB2HADRDBNAME?}" \
   | grep ${grepFlags?} Role | grep ${grepFlags?} State \
   | grep "[a-zA-Z]" \
   | tail -1 | awk '{print $1 "\n" $2}' > $temp_snap

   if [ -r $temp_snap ]; then
      hadr_role=$(head -1 $temp_snap)
      hadr_state=$(tail -1 $temp_snap)
   fi
   rm -f $temp_snap

   # if we did not retrieve any db2pd info (even though we would expect to), must use SNAPSHOT
   if [[ -z $hadr_role ]]; then
      if [[ ! -z $hadr_process ]]; then
         # Any problems with db2pd output, we'll resort to snapshot API
         logger -i -p debug -t $0 "$DB2HADRDBNAME : Nothing returned from db2pd, using SNAPSHOT"
         su - ${instance_to_monitor} -c "db2 get snapshot for all on ${DB2HADRDBNAME?}" \
         | egrep "(^ [ ]+State[ ]+=[ ]+)|(^[ ]+Role[ ]+=[ ]+)" \
         | sort | awk '{print $3}' > $temp_snap2

         if [ -r $temp_snap2 ]; then
            hadr_role=$(head -1 $temp_snap2)
            hadr_state=$(tail -1 $temp_snap2)
         fi
      fi
   fi
   rm -f $temp_snap2

   # hadr_role & hadr_state should now be set

   if [[ $hadr_role == "Primary" ]]; then
      if [[ $hadr_state == "Peer" ]]; then
         # Online 
         rgreq -o Unlock $DATA_SVC_RG_NAME 2> /dev/null
         rc=1
      else
         # Primary non-Peer
         logger -i -p notice -t $0 "$DB2HADRDBNAME ($hadr_role, $hadr_state). Locking resource $DATA_SVC_RG_NAME"
         rgreq -o Lock $DATA_SVC_RG_NAME 2> /dev/null
         if [[ "$monitor_as_standby" == "N" ]]; then
            # Return online 
            rc=1
         else
            # Return Primary Non Peer distinctly
            rc=3
         fi
      fi

   elif [[ $hadr_role == "Standby" ]]; then
      if [[ $hadr_state == "Peer" ]]; then
         rc=2
      else
         # Must return Standby states as offline to TSA
         # Only if script called directly with S option
         #  is the standby states returned distinctly
         if [[ "$monitor_as_standby" == "N" ]]; then
            # Standby return as Offline
            rc=2
         else
            rc=40
         fi
      fi
   else
      # Unknown state
      if [[ "$monitor_as_standby" == "N" ]]; then

         lockreq=$(/usr/bin/samdiag -x votes -g ${DATA_SVC_RG_NAME?} 2> /dev/null | grep -c "Type: lock")
         if [[ $lockreq -eq 0 ]]; then
            logger -i -p notice -t $0 "$DATA_SVC_RG_NAME state is not known, returning Unknown: $lockreq"
            rc=0
         else
            logger -i -p notice -t $0 "$DATA_SVC_RG_NAME state is not known, $DATA_SVC_RG_NAME is locked, returing Offline: $lockreq"
            rc=2
         fi
      else
         rc=50
      fi
   fi

   return $rc
}

###########################################################
# probehadr()
#
# Probe status of HADR on the active resource on this node
#
###########################################################
probehadr()
{
   unset instance_to_monitor
   set_candidate_P_instance
   if [[ -z "$candidate_P_instance" ]]; then
      # If we cannot find instance to monitor, return Unknown
      logger -i -p err -t $0 "Cannot find instance name!"
      exit 0
   fi
   instance_to_monitor=$candidate_P_instance
   probehadr_on_this_node
}

#######################################################
# main()
#######################################################
main()
{

   probehadr
   if [ -f /tmp/.virtual_offline_${CLUSTER_APP_LABEL?} ]; then
      # Offline
      rc=2
   fi

   logger -i -p info -t $0 "$DB2HADRDBNAME ($hadr_role, $hadr_state). Returning $rc"
   return $rc
}

# 

main "${@:-}"

echo $rc
exit $rc
