#!/bin/ksh -p
#-----------------------------------------------------------------------
# (C) COPYRIGHT International Business Machines Corp. 2001-2006
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# NAME: hadr_start.ksh
#
# (%W%) %E%  %U%
#
# FUNCTION: Start DB2 UDB, for specified RESOURCE_NAME and RESOURCE_GROUP
#
# INPUT:  hadr_start.ksh db2instp db2insts db2hadrdb verbose [S]
#
# OUTPUT: 0 if started ok
#
#-----------------------------------------------------------------------
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

logger -i -p info -t $0 "$*"

RESOURCE_METHOD=start

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
DB2HADRINSTANCE1=${1?}
DB2HADRINSTANCE2=${2?}
DB2HADRDBNAME=${3?}
VERBOSE=${4:-noverbose}
start_as_standby=${5:-N}

export CT_MANAGEMENT_SCOPE=2

if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
   set -x
else
   # Close stdout and stderr
   exec 2> /dev/null
   exec 1> /dev/null
   set +x
fi

PROBE=
START=$PROGNAME
STOP=
RT_BASEDIR=${PROGPATH?}
SVC_PROBE=${RT_BASEDIR}/${PROBE:-hadr_monitor.ksh}
START_CMD=$RT_BASEDIR/${START:-hadr_start.ksh}
STOP_CMD=$RT_BASEDIR/${STOP:-hadr_stop.ksh}

if [[ $DB2HADRINSTANCE1 != $DB2HADRINSTANCE2 ]]; then
   RGNAME1=db2_${DB2HADRINSTANCE1?}_
   RSNAME1=db2_${DB2HADRINSTANCE1?}_
   RGNAME2=db2_${DB2HADRINSTANCE2?}_
   RSNAME2=db2_${DB2HADRINSTANCE2?}_

   Resource=${RSNAME1?}
   ResourceClass=IBM.Application
   NodeRG1=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | tail -1)

   Resource=${RSNAME2?}
   ResourceClass=IBM.Application
   NodeRG2=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | tail -1)

else
   Resource=db2_${DB2HADRINSTANCE1?}_
   ResourceClass=IBM.Application

   NodeRG1=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList  | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | sort -n | uniq | head -1)
   NodeRG2=$(lsrsrc-api -s ${ResourceClass?}::'Name like "'${Resource?}%'"'::NodeNameList  | tr "{" " " | tr "}" " " | tr "." " " | awk '{print $1}' | sort -n | uniq | tail -1)

   RGNAME1=db2_${DB2HADRINSTANCE1?}_${NodeRG1?}_0-rg
   RSNAME1=db2_${DB2HADRINSTANCE1?}_${NodeRG1?}_0-rs
   RGNAME2=db2_${DB2HADRINSTANCE2?}_${NodeRG2?}_0-rg
   RSNAME2=db2_${DB2HADRINSTANCE2?}_${NodeRG2?}_0-rs
fi
CLUSTER_APP_LABEL=db2hadr_${DB2HADRDBNAME?}

# Name the resource group and the resource
DATA_SVC_RG_NAME=${CLUSTER_APP_LABEL}-rg
DATA_SVC_R_NAME=${CLUSTER_APP_LABEL}-rs


##########################################################
# error_exit
##########################################################
error_exit()
{
   typeset exit_code="${rc:-1}"
   logger -i  -p err -t $0 "exiting with $rc"
   exit $rc
}

###########################################################
# resource_group_state_on_this_node()
#
# INPUT: ResourceClass, Resource, ResourceType, Node 
# OUTPUT:rg_state [Online=1 || Offline]
#
###########################################################
resource_group_state_on_this_node()
{
   lsrsrc-api -s ${ResourceClass}::'Name="'${Resource}'" '"${ResourceType}"' '::NodeNameList 

}

###########################################################
# HADR_partner_node_state()
#
# INPUT: HADR_REMOTE_HOST [hostname]
# OUTPUT:remote_node_alive [Online || Offline]
#
###########################################################
HADR_partner_node_state()
{
   remote_node_alive=Online

   if [[ ! -z $HADR_REMOTE_HOST ]]; then
      remote_host_in_tsa_format=$(echo ${HADR_REMOTE_HOST?} | tr "." " " | awk '{print $1}')
      remote_node_alive=$(lsrpnode -x  | sort | awk '{print $1 " " $2}' | grep ${remote_host_in_tsa_format?} | awk '{print $2}')
   else
      # Is any node in domain offline ...
      nno=$(lsrpnode -x -O | wc -l)
      if [[ $nno == 0 ]]; then
         remote_node_alive=Online
      else
         remote_node_alive=Offline
      fi
   fi
}

###########################################################
# set_candidate_P_instance()
#
# INPUT: RSNAME1, RSNAME2, DB2HADRINSTANCE1, DB2HADRINTANCE2
# OUTPUT:candidate_P_instance
#
###########################################################
set_candidate_P_instance()
{
   Node=$(hostname | tr "." " " | awk '{print $1}')

   if   [[ $NodeRG1 == $Node && $NodeRG2 != $Node ]]; then
      candidate_P_instance=$DB2HADRINSTANCE1
      HADR_REMOTE_HOST=$NodeRG2
      forceRGOfflineInCaseOfByForce=$RGNAME2

   elif [[ $NodeRG1 != $Node && $NodeRG2 == $Node ]]; then
      candidate_P_instance=$DB2HADRINSTANCE2
      HADR_REMOTE_HOST=$NodeRG1
      forceRGOfflineInCaseOfByForce=$RGNAME1
   else
      # either both instances are up in resource groups
      # on this node, or neither is
      # figure out the one that wants to be primary
      HADR_LOCAL_HOST1=$(su - ${DB2HADRINSTANCE1?} -c "db2 get db cfg for ${DB2HADRDBNAME?}" | grep HADR_LOCAL_HOST | awk '{print $7}' | tr "." " " | awk '{print $1}' | tail -1)
      HADR_LOCAL_HOST2=$(su - ${DB2HADRINSTANCE2?} -c "db2 get db cfg for ${DB2HADRDBNAME?}" | grep HADR_LOCAL_HOST | awk '{print $7}' | tr "." " " | awk '{print $1}' | tail -1)
      if [[ $HADR_LOCAL_HOST1 == $Node ]]; then
         candidate_P_instance=$DB2HADRINSTANCE1
         forceRGOfflineInCaseOfByForce=$RGNAME2
         HADR_REMOTE_HOST=$NodeRG2
      elif [[ $HADR_LOCAL_HOST2 == $Node ]]; then
         candidate_P_instance=$DB2HADRINSTANCE2
         forceRGOfflineInCaseOfByForce=$RGNAME1
         HADR_REMOTE_HOST=$NodeRG1
      else
         candidate_P_instance=
         forceRGOfflineInCaseOfByForce=
         HADR_REMOTE_HOST=
      fi
   fi
}

###########################################################
# starthadr()
###########################################################
starthadr()
{
   set_candidate_P_instance
   instance_to_start=${candidate_P_instance}

   $SVC_PROBE ${DB2HADRINSTANCE1?} ${DB2HADRINSTANCE2?} ${DB2HADRDBNAME?} ${VERBOSE?} S
   rc=$?

   HADR_partner_node_state

   if [[ $remote_node_alive == "Online" ]]; then
      # Bring up HADR as Primary on this node
      if [ $rc -eq 1 ]; then
         # already primary
         rc=0
      elif [ $rc -eq 2 ]; then
         # currently standby,peer
         # takeover (no force)
         logger -i  -p notice -t $0 "su - ${instance_to_start?} -c db2 takeover hadr on db ${DB2HADRDBNAME?}"
         su - ${instance_to_start?} -c "db2 takeover hadr on db ${DB2HADRDBNAME?}" 
         $SVC_PROBE ${DB2HADRINSTANCE1?} ${DB2HADRINSTANCE2?} ${DB2HADRDBNAME?} ${VERBOSE?}
         rc1=$?
         if [ $rc1 -ne 1 ]; then
            :
            logger -i  -p err -t $0 "*** Database ${DB2HADRDBNAME} is in Peer State, TAKEOVER FAILED"
            # Old primary node is still online, offline instance to prevent split-brain
            # Uncomment following 3 lines to allow takeover by force
            #chrg -o Offline -s "Name = '${forceRGOfflineInCaseOfByForce?}'"
            #su - ${instance_to_start?} -c "db2 takeover hadr on db ${DB2HADRDBNAME?} by force"
            #logger -i  -p notice -t $0 "NOTICE: Takeover by force issued, old primary instance offlined to prevent split brain"
         fi

      elif [ $rc -eq 40 ]; then
         :
         logger -i  -p err -t $0 "*** Database ${DB2HADRDBNAME} is not in Peer State, old Primary machine still Online"
         # Uncomment following 3 lines to allow takeover even in case of non Peer Standby w/ old Primary machine Online
         #chrg -o Offline -s "Name = '${forceRGOfflineInCaseOfByForce?}'"
         #su - ${instance_to_start?} -c "db2 takeover hadr on db ${DB2HADRDBNAME?} by force "
         #logger -i  -p notice -t $0 "NOTICE: Takeover by force issued, old primary instance offlined to prevent split brain"
      else
         # current state of HADR is unknown
         # eg. If instance has just gone down, wait until it's 2*monitor period 
         # so that instance can be restarted and db ACTIVATEd
         sleep 20
      fi # Bring up HADR as Primary on this machine

   else
      # Old primary machine is offline
      if [ $rc -eq 2 ]; then
         # Standby is currently in Peer State
         #
         # To bring up standby, will now do a TAKEOVER BY FORCE
         # No need to block until resource group is offline, we have verified
         # that the node is down already
         # To bring up standby, will now do a TAKEOVER BY FORCE
         :
         logger -i  -p notice -t $0 "su - ${instance_to_start?} -c db2 takeover hadr on db ${DB2HADRDBNAME?} by force"
         su - ${instance_to_start?} -c "db2 takeover hadr on db ${DB2HADRDBNAME?} by force "
         logger -i  -p notice -t $0 "NOTICE: Takeover by force issued"
      elif [ $rc -eq 40 ]; then
         # Standby is currently not in Peer State
         :
         logger -i  -p err -t $0 "*** Database ${DB2HADRDBNAME} is not in Peer State, old Primary machine Offline"
         # Uncomment following 3 lines to allow takeover even in case of non Peer Standby w/ old Primary machine Offline
         logger -i  -p notice -t $0 "su - ${instance_to_start?} -c db2 takeover hadr on db ${DB2HADRDBNAME?} by force"
         su - ${instance_to_start?} -c "db2 takeover hadr on db ${DB2HADRDBNAME?} by force "
         logger -i  -p notice -t $0 "NOTICE: Takeover by force issued"
      fi
   fi # Bring up HADR on this machine

   # Return state 
   $SVC_PROBE ${DB2HADRINSTANCE1?} ${DB2HADRINSTANCE2?} ${DB2HADRDBNAME?} ${VERBOSE?} S
   rcs=$?

   # Online succesful must return 0 whilst monitor returns 1 
   # for Primary in Peer State and 3 for Primary not Peer
   if [ $rcs -eq 1 ]; then
      rc=0
   elif [ $rcs -eq 3 ]; then
      rc=0
      # Anything else, map directly from monitor
   else
      rc=$rcs
   fi

   return $rc
}

#######################################################
# main()
#######################################################
main()
{
   rm -f /tmp/.virtual_offline_${CLUSTER_APP_LABEL?}
   starthadr
   logger -i -p info -t $0 "$CLUSTER_APP_LABEL returning $rc"
   return $rc
}

#
if [[ "$VERBOSE" == "verbose" ]]; then
   typeset -ft $(typeset +f)
fi

main "${@:-}"

echo $rc
exit $rc
